From af1bd8acd50ccf82298460145f3c575ea2c63925 Mon Sep 17 00:00:00 2001
From: Oleksandr Shulgin <oleksandr.shulgin@zalando.de>
Date: Thu, 20 Aug 2015 09:48:13 +0200
Subject: [PATCH 4/5] Add TSConfigMap support to DDL deparse.

---
 src/backend/catalog/dependency.c             |  5 ++
 src/backend/catalog/objectaddress.c          | 31 ++++++++-
 src/backend/tcop/utility.c                   |  2 +
 src/include/catalog/catversion.h             |  2 +-
 src/include/catalog/dependency.h             |  1 +
 src/include/nodes/parsenodes.h               |  1 +
 src/test/regress/expected/object_address.out | 98 +++++++++++++++-------------
 src/test/regress/sql/object_address.sql      |  3 +
 8 files changed, 97 insertions(+), 46 deletions(-)

diff --git a/src/backend/catalog/dependency.c b/src/backend/catalog/dependency.c
index 90b1cd8..08a3ea6 100644
--- a/src/backend/catalog/dependency.c
+++ b/src/backend/catalog/dependency.c
@@ -50,6 +50,7 @@
 #include "catalog/pg_transform.h"
 #include "catalog/pg_trigger.h"
 #include "catalog/pg_ts_config.h"
+#include "catalog/pg_ts_config_map.h"
 #include "catalog/pg_ts_dict.h"
 #include "catalog/pg_ts_parser.h"
 #include "catalog/pg_ts_template.h"
@@ -149,6 +150,7 @@ static const Oid object_classes[] = {
 	TSDictionaryRelationId,		/* OCLASS_TSDICT */
 	TSTemplateRelationId,		/* OCLASS_TSTEMPLATE */
 	TSConfigRelationId,			/* OCLASS_TSCONFIG */
+	TSConfigMapRelationId,		/* OCLASS_TSCONFIG_MAP */
 	AuthIdRelationId,			/* OCLASS_ROLE */
 	DatabaseRelationId,			/* OCLASS_DATABASE */
 	TableSpaceRelationId,		/* OCLASS_TBLSPACE */
@@ -2382,6 +2384,9 @@ getObjectClass(const ObjectAddress *object)
 		case TSConfigRelationId:
 			return OCLASS_TSCONFIG;
 
+		case TSConfigMapRelationId:
+			return OCLASS_TSCONFIG_MAP;
+
 		case AuthIdRelationId:
 			return OCLASS_ROLE;
 
diff --git a/src/backend/catalog/objectaddress.c b/src/backend/catalog/objectaddress.c
index 052aab1..e01caa1 100644
--- a/src/backend/catalog/objectaddress.c
+++ b/src/backend/catalog/objectaddress.c
@@ -48,6 +48,7 @@
 #include "catalog/pg_transform.h"
 #include "catalog/pg_trigger.h"
 #include "catalog/pg_ts_config.h"
+#include "catalog/pg_ts_config_map.h"
 #include "catalog/pg_ts_dict.h"
 #include "catalog/pg_ts_parser.h"
 #include "catalog/pg_ts_template.h"
@@ -390,6 +391,18 @@ static const ObjectPropertyType ObjectProperty[] =
 		true
 	},
 	{
+		TSConfigMapRelationId,
+		TSConfigMapIndexId,
+		TSCONFIGMAP,
+		TSCONFIGNAMENSP,
+		InvalidAttrNumber,
+		InvalidAttrNumber,
+		InvalidAttrNumber,
+		InvalidAttrNumber,
+		ACL_KIND_TSCONFIGURATION,
+		true
+	},
+	{
 		TSDictionaryRelationId,
 		TSDictionaryOidIndexId,
 		TSDICTOID,
@@ -595,6 +608,10 @@ static const struct object_type_map
 	{
 		"text search configuration", OBJECT_TSCONFIGURATION
 	},
+	/* OCLASS_TSCONFIG_MAP */
+	{
+		"text search configuration mapping", OBJECT_TSCONFIG_MAPPING
+	},
 	/* OCLASS_ROLE */
 	{
 		"role", OBJECT_ROLE
@@ -903,6 +920,11 @@ get_object_address(ObjectType objtype, List *objname, List *objargs,
 				address.objectId = get_ts_config_oid(objname, missing_ok);
 				address.objectSubId = 0;
 				break;
+			case OBJECT_TSCONFIG_MAPPING:
+				address.classId = TSConfigMapRelationId;
+				address.objectId = get_ts_config_oid(objname, missing_ok);
+				address.objectSubId = 0;
+				break;
 			case OBJECT_USER_MAPPING:
 				address = get_object_address_usermapping(objname, objargs,
 														 missing_ok);
@@ -2150,6 +2172,7 @@ check_object_ownership(Oid roleid, ObjectType objtype, ObjectAddress address,
 							   NameListToString(objname));
 			break;
 		case OBJECT_TSCONFIGURATION:
+		case OBJECT_TSCONFIG_MAPPING:
 			if (!pg_ts_config_ownercheck(address.objectId, roleid))
 				aclcheck_error(ACLCHECK_NOT_OWNER, ACL_KIND_TSCONFIGURATION,
 							   NameListToString(objname));
@@ -2237,7 +2260,7 @@ get_object_namespace(const ObjectAddress *address)
 int
 read_objtype_from_string(const char *objtype)
 {
-	ObjectType	type;
+	ObjectType	type = -1;
 	int			i;
 
 	for (i = 0; i < lengthof(ObjectTypeMap); i++)
@@ -2905,6 +2928,7 @@ getObjectDescription(const ObjectAddress *object)
 			}
 
 		case OCLASS_TSCONFIG:
+		case OCLASS_TSCONFIG_MAP:
 			{
 				HeapTuple	tup;
 
@@ -3564,6 +3588,10 @@ getObjectTypeDescription(const ObjectAddress *object)
 			appendStringInfoString(&buffer, "text search configuration");
 			break;
 
+		case OCLASS_TSCONFIG_MAP:
+			appendStringInfoString(&buffer, "text search configuration mapping");
+			break;
+
 		case OCLASS_ROLE:
 			appendStringInfoString(&buffer, "role");
 			break;
@@ -4296,6 +4324,7 @@ getObjectIdentityParts(const ObjectAddress *object,
 			}
 
 		case OCLASS_TSCONFIG:
+		case OCLASS_TSCONFIG_MAP:
 			{
 				HeapTuple	tup;
 				Form_pg_ts_config formCfg;
diff --git a/src/backend/tcop/utility.c b/src/backend/tcop/utility.c
index e81bbc6..b848ec6 100644
--- a/src/backend/tcop/utility.c
+++ b/src/backend/tcop/utility.c
@@ -1454,6 +1454,8 @@ ProcessUtilitySlow(Node *parsetree,
 
 			case T_AlterTSConfigurationStmt:
 				address = AlterTSConfiguration((AlterTSConfigurationStmt *) parsetree);
+				/* commands are stashed in Make/DropConfigurationMapping */
+				commandCollected = true;
 				break;
 
 			case T_AlterTableMoveAllStmt:
diff --git a/src/include/catalog/catversion.h b/src/include/catalog/catversion.h
index b58fe46..2be1aba 100644
--- a/src/include/catalog/catversion.h
+++ b/src/include/catalog/catversion.h
@@ -53,6 +53,6 @@
  */
 
 /*							yyyymmddN */
-#define CATALOG_VERSION_NO	201508111
+#define CATALOG_VERSION_NO	201508211
 
 #endif
diff --git a/src/include/catalog/dependency.h b/src/include/catalog/dependency.h
index fbcf904..b400095 100644
--- a/src/include/catalog/dependency.h
+++ b/src/include/catalog/dependency.h
@@ -143,6 +143,7 @@ typedef enum ObjectClass
 	OCLASS_TSDICT,				/* pg_ts_dict */
 	OCLASS_TSTEMPLATE,			/* pg_ts_template */
 	OCLASS_TSCONFIG,			/* pg_ts_config */
+	OCLASS_TSCONFIG_MAP,		/* pg_ts_config_map */
 	OCLASS_ROLE,				/* pg_authid */
 	OCLASS_DATABASE,			/* pg_database */
 	OCLASS_TBLSPACE,			/* pg_tablespace */
diff --git a/src/include/nodes/parsenodes.h b/src/include/nodes/parsenodes.h
index 151c93a..a151412 100644
--- a/src/include/nodes/parsenodes.h
+++ b/src/include/nodes/parsenodes.h
@@ -1408,6 +1408,7 @@ typedef enum ObjectType
 	OBJECT_TRANSFORM,
 	OBJECT_TRIGGER,
 	OBJECT_TSCONFIGURATION,
+	OBJECT_TSCONFIG_MAPPING,
 	OBJECT_TSDICTIONARY,
 	OBJECT_TSPARSER,
 	OBJECT_TSTEMPLATE,
diff --git a/src/test/regress/expected/object_address.out b/src/test/regress/expected/object_address.out
index 034c0b1..0da2a34 100644
--- a/src/test/regress/expected/object_address.out
+++ b/src/test/regress/expected/object_address.out
@@ -12,6 +12,7 @@ CREATE FOREIGN DATA WRAPPER addr_fdw;
 CREATE SERVER addr_fserv FOREIGN DATA WRAPPER addr_fdw;
 CREATE TEXT SEARCH DICTIONARY addr_ts_dict (template=simple);
 CREATE TEXT SEARCH CONFIGURATION addr_ts_conf (copy=english);
+ALTER TEXT SEARCH CONFIGURATION addr_ts_conf ALTER MAPPING FOR asciiword WITH addr_ts_dict;
 CREATE TEXT SEARCH TEMPLATE addr_ts_temp (lexize=dsimple_lexize);
 CREATE TEXT SEARCH PARSER addr_ts_prs
     (start = prsd_start, gettoken = prsd_nexttoken, end = prsd_end, lextypes = prsd_lextype);
@@ -79,6 +80,7 @@ BEGIN
 		('operator'), ('operator class'), ('operator family'), ('rule'), ('trigger'),
 		('text search parser'), ('text search dictionary'),
 		('text search template'), ('text search configuration'),
+		('text search configuration mapping'),
 		('policy'), ('user mapping'), ('default acl'), ('transform'),
 		('operator of access method'), ('function of access method')
 	LOOP
@@ -246,6 +248,12 @@ WARNING:  error for text search configuration,{addr_nsp,zwei},{}: text search co
 WARNING:  error for text search configuration,{addr_nsp,zwei},{integer}: text search configuration "addr_nsp.zwei" does not exist
 WARNING:  error for text search configuration,{eins,zwei,drei},{}: cross-database references are not implemented: eins.zwei.drei
 WARNING:  error for text search configuration,{eins,zwei,drei},{integer}: cross-database references are not implemented: eins.zwei.drei
+WARNING:  error for text search configuration mapping,{eins},{}: text search configuration "eins" does not exist
+WARNING:  error for text search configuration mapping,{eins},{integer}: text search configuration "eins" does not exist
+WARNING:  error for text search configuration mapping,{addr_nsp,zwei},{}: text search configuration "addr_nsp.zwei" does not exist
+WARNING:  error for text search configuration mapping,{addr_nsp,zwei},{integer}: text search configuration "addr_nsp.zwei" does not exist
+WARNING:  error for text search configuration mapping,{eins,zwei,drei},{}: cross-database references are not implemented: eins.zwei.drei
+WARNING:  error for text search configuration mapping,{eins,zwei,drei},{integer}: cross-database references are not implemented: eins.zwei.drei
 WARNING:  error for policy,{eins},{}: must specify relation and object name
 WARNING:  error for policy,{eins},{integer}: must specify relation and object name
 WARNING:  error for policy,{addr_nsp,zwei},{}: relation "addr_nsp" does not exist
@@ -362,6 +370,7 @@ WITH objects (type, name, args) AS (VALUES
 				('text search dictionary', '{addr_ts_dict}', '{}'),
 				('text search template', '{addr_ts_temp}', '{}'),
 				('text search configuration', '{addr_ts_conf}', '{}'),
+				('text search configuration mapping', '{addr_ts_conf}', '{}'),
 				('role', '{regtest_addr_user}', '{}'),
 				-- database
 				-- tablespace
@@ -383,50 +392,51 @@ SELECT (pg_identify_object(addr1.classid, addr1.objid, addr1.subobjid)).*,
 			pg_identify_object_as_address(classid, objid, subobjid) ioa(typ,nms,args),
 			pg_get_object_address(typ, nms, ioa.args) as addr2
 	ORDER BY addr1.classid, addr1.objid, addr1.subobjid;
-           type            |   schema   |       name        |                               identity                               | ?column? 
----------------------------+------------+-------------------+----------------------------------------------------------------------+----------
- default acl               |            |                   | for role regtest_addr_user in schema public on tables                | t
- default acl               |            |                   | for role regtest_addr_user on tables                                 | t
- type                      | pg_catalog | _int4             | integer[]                                                            | t
- type                      | addr_nsp   | gencomptype       | addr_nsp.gencomptype                                                 | t
- type                      | addr_nsp   | genenum           | addr_nsp.genenum                                                     | t
- type                      | addr_nsp   | gendomain         | addr_nsp.gendomain                                                   | t
- function                  | pg_catalog |                   | pg_catalog.pg_identify_object(pg_catalog.oid,pg_catalog.oid,integer) | t
- aggregate                 | addr_nsp   |                   | addr_nsp.genaggr(integer)                                            | t
- sequence                  | addr_nsp   | gentable_a_seq    | addr_nsp.gentable_a_seq                                              | t
- table                     | addr_nsp   | gentable          | addr_nsp.gentable                                                    | t
- table column              | addr_nsp   | gentable          | addr_nsp.gentable.b                                                  | t
- index                     | addr_nsp   | gentable_pkey     | addr_nsp.gentable_pkey                                               | t
- view                      | addr_nsp   | genview           | addr_nsp.genview                                                     | t
- materialized view         | addr_nsp   | genmatview        | addr_nsp.genmatview                                                  | t
- foreign table             | addr_nsp   | genftable         | addr_nsp.genftable                                                   | t
- foreign table column      | addr_nsp   | genftable         | addr_nsp.genftable.a                                                 | t
- role                      |            | regtest_addr_user | regtest_addr_user                                                    | t
- server                    |            | addr_fserv        | addr_fserv                                                           | t
- user mapping              |            |                   | regtest_addr_user on server integer                                  | t
- foreign-data wrapper      |            | addr_fdw          | addr_fdw                                                             | t
- operator of access method |            |                   | operator 1 (integer, integer) of pg_catalog.integer_ops USING btree  | t
- function of access method |            |                   | function 2 (integer, integer) of pg_catalog.integer_ops USING btree  | t
- default value             |            |                   | for addr_nsp.gentable.b                                              | t
- cast                      |            |                   | (bigint AS integer)                                                  | t
- table constraint          | addr_nsp   |                   | a_chk on addr_nsp.gentable                                           | t
- domain constraint         | addr_nsp   |                   | domconstr on addr_nsp.gendomain                                      | t
- conversion                | pg_catalog | ascii_to_mic      | pg_catalog.ascii_to_mic                                              | t
- language                  |            | plpgsql           | plpgsql                                                              | t
- schema                    |            | addr_nsp          | addr_nsp                                                             | t
- operator class            | pg_catalog | int4_ops          | pg_catalog.int4_ops USING btree                                      | t
- operator                  | pg_catalog |                   | pg_catalog.+(integer,integer)                                        | t
- rule                      |            |                   | "_RETURN" on addr_nsp.genview                                        | t
- trigger                   |            |                   | t on addr_nsp.gentable                                               | t
- operator family           | pg_catalog | integer_ops       | pg_catalog.integer_ops USING btree                                   | t
- policy                    |            |                   | genpol on addr_nsp.gentable                                          | t
- collation                 | pg_catalog | "default"         | pg_catalog."default"                                                 | t
- transform                 |            |                   | for integer on language sql                                          | t
- text search dictionary    | addr_nsp   | addr_ts_dict      | addr_nsp.addr_ts_dict                                                | t
- text search parser        | addr_nsp   | addr_ts_prs       | addr_nsp.addr_ts_prs                                                 | t
- text search configuration | addr_nsp   | addr_ts_conf      | addr_nsp.addr_ts_conf                                                | t
- text search template      | addr_nsp   | addr_ts_temp      | addr_nsp.addr_ts_temp                                                | t
-(41 rows)
+               type                |   schema   |       name        |                               identity                               | ?column? 
+-----------------------------------+------------+-------------------+----------------------------------------------------------------------+----------
+ default acl                       |            |                   | for role regtest_addr_user in schema public on tables                | t
+ default acl                       |            |                   | for role regtest_addr_user on tables                                 | t
+ type                              | pg_catalog | _int4             | integer[]                                                            | t
+ type                              | addr_nsp   | gencomptype       | addr_nsp.gencomptype                                                 | t
+ type                              | addr_nsp   | genenum           | addr_nsp.genenum                                                     | t
+ type                              | addr_nsp   | gendomain         | addr_nsp.gendomain                                                   | t
+ function                          | pg_catalog |                   | pg_catalog.pg_identify_object(pg_catalog.oid,pg_catalog.oid,integer) | t
+ aggregate                         | addr_nsp   |                   | addr_nsp.genaggr(integer)                                            | t
+ sequence                          | addr_nsp   | gentable_a_seq    | addr_nsp.gentable_a_seq                                              | t
+ table                             | addr_nsp   | gentable          | addr_nsp.gentable                                                    | t
+ table column                      | addr_nsp   | gentable          | addr_nsp.gentable.b                                                  | t
+ index                             | addr_nsp   | gentable_pkey     | addr_nsp.gentable_pkey                                               | t
+ view                              | addr_nsp   | genview           | addr_nsp.genview                                                     | t
+ materialized view                 | addr_nsp   | genmatview        | addr_nsp.genmatview                                                  | t
+ foreign table                     | addr_nsp   | genftable         | addr_nsp.genftable                                                   | t
+ foreign table column              | addr_nsp   | genftable         | addr_nsp.genftable.a                                                 | t
+ role                              |            | regtest_addr_user | regtest_addr_user                                                    | t
+ server                            |            | addr_fserv        | addr_fserv                                                           | t
+ user mapping                      |            |                   | regtest_addr_user on server integer                                  | t
+ foreign-data wrapper              |            | addr_fdw          | addr_fdw                                                             | t
+ operator of access method         |            |                   | operator 1 (integer, integer) of pg_catalog.integer_ops USING btree  | t
+ function of access method         |            |                   | function 2 (integer, integer) of pg_catalog.integer_ops USING btree  | t
+ default value                     |            |                   | for addr_nsp.gentable.b                                              | t
+ cast                              |            |                   | (bigint AS integer)                                                  | t
+ table constraint                  | addr_nsp   |                   | a_chk on addr_nsp.gentable                                           | t
+ domain constraint                 | addr_nsp   |                   | domconstr on addr_nsp.gendomain                                      | t
+ conversion                        | pg_catalog | ascii_to_mic      | pg_catalog.ascii_to_mic                                              | t
+ language                          |            | plpgsql           | plpgsql                                                              | t
+ schema                            |            | addr_nsp          | addr_nsp                                                             | t
+ operator class                    | pg_catalog | int4_ops          | pg_catalog.int4_ops USING btree                                      | t
+ operator                          | pg_catalog |                   | pg_catalog.+(integer,integer)                                        | t
+ rule                              |            |                   | "_RETURN" on addr_nsp.genview                                        | t
+ trigger                           |            |                   | t on addr_nsp.gentable                                               | t
+ operator family                   | pg_catalog | integer_ops       | pg_catalog.integer_ops USING btree                                   | t
+ policy                            |            |                   | genpol on addr_nsp.gentable                                          | t
+ collation                         | pg_catalog | "default"         | pg_catalog."default"                                                 | t
+ transform                         |            |                   | for integer on language sql                                          | t
+ text search dictionary            | addr_nsp   | addr_ts_dict      | addr_nsp.addr_ts_dict                                                | t
+ text search parser                | addr_nsp   | addr_ts_prs       | addr_nsp.addr_ts_prs                                                 | t
+ text search configuration         | addr_nsp   | addr_ts_conf      | addr_nsp.addr_ts_conf                                                | t
+ text search configuration mapping |            |                   | addr_nsp.addr_ts_conf                                                | t
+ text search template              | addr_nsp   | addr_ts_temp      | addr_nsp.addr_ts_temp                                                | t
+(42 rows)
 
 ---
 --- Cleanup resources
diff --git a/src/test/regress/sql/object_address.sql b/src/test/regress/sql/object_address.sql
index 68e7cb0..a4f0aee 100644
--- a/src/test/regress/sql/object_address.sql
+++ b/src/test/regress/sql/object_address.sql
@@ -16,6 +16,7 @@ CREATE FOREIGN DATA WRAPPER addr_fdw;
 CREATE SERVER addr_fserv FOREIGN DATA WRAPPER addr_fdw;
 CREATE TEXT SEARCH DICTIONARY addr_ts_dict (template=simple);
 CREATE TEXT SEARCH CONFIGURATION addr_ts_conf (copy=english);
+ALTER TEXT SEARCH CONFIGURATION addr_ts_conf ALTER MAPPING FOR asciiword WITH addr_ts_dict;
 CREATE TEXT SEARCH TEMPLATE addr_ts_temp (lexize=dsimple_lexize);
 CREATE TEXT SEARCH PARSER addr_ts_prs
     (start = prsd_start, gettoken = prsd_nexttoken, end = prsd_end, lextypes = prsd_lextype);
@@ -77,6 +78,7 @@ BEGIN
 		('operator'), ('operator class'), ('operator family'), ('rule'), ('trigger'),
 		('text search parser'), ('text search dictionary'),
 		('text search template'), ('text search configuration'),
+		('text search configuration mapping'),
 		('policy'), ('user mapping'), ('default acl'), ('transform'),
 		('operator of access method'), ('function of access method')
 	LOOP
@@ -155,6 +157,7 @@ WITH objects (type, name, args) AS (VALUES
 				('text search dictionary', '{addr_ts_dict}', '{}'),
 				('text search template', '{addr_ts_temp}', '{}'),
 				('text search configuration', '{addr_ts_conf}', '{}'),
+				('text search configuration mapping', '{addr_ts_conf}', '{}'),
 				('role', '{regtest_addr_user}', '{}'),
 				-- database
 				-- tablespace
-- 
2.1.4

